static const char* const rcsid = "$Id: send-message.c,v 1.1 2003/10/21 08:12:17 felix Exp $";

/*                               -*- Mode: C -*- 
 * send-message.c -- 
 * 
 * Copyright (c) 1991-2003 Francois Felix Ingrand.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "config.h"


#ifdef VXWORKS
#include "vxWorks.h"
#include "ioLib.h"
#include "in.h"
#include "socket.h"
#else
#ifdef WIN95
#include "winsock.h"
#else
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <signal.h>
#include <netinet/in.h>
#include <netdb.h>
#endif
#endif

#include <errno.h>
#include <string.h>

#include "constant.h"
#include "macro.h"

#include "oprs-type.h"
#include "mp-register.h"
#include "oprs-profiling.h"

#ifdef OPRS_PROFILING
#include "user-external.h"
#endif

#include "send-message_f.h"
#include "oprs-profiling_f.h"
#include "mp-register_f.h"

void send_message_string_socket(int socket, PString rec, PString message )
{
     int h_size_name,i,total_size, h_size_mess;
     u_long n_size_name, n_size_mess;                  /* for network */
     Message_Type mt = MESSAGE_MT;
     u_long n_mt;

     char *buf, *tmp;

     h_size_name = strlen(rec);
     h_size_mess = strlen(message);
     n_size_name = htonl(h_size_name);     
     n_size_mess = htonl(h_size_mess);     
     n_mt = htonl(mt);

     total_size = sizeof(n_mt) + sizeof(n_size_name) + h_size_name + sizeof(n_size_mess) + h_size_mess;

     tmp = buf = (char *)malloc(total_size);
     
     BCOPY((char *)&n_mt,tmp, sizeof(n_mt));
     tmp +=  sizeof(n_mt);

     BCOPY((char *)&n_size_name,tmp, sizeof(n_size_name));
     tmp +=  sizeof(n_size_name);
     BCOPY((char *)rec,tmp, h_size_name);
     tmp +=  h_size_name;

     BCOPY((char *)&n_size_mess,tmp, sizeof(n_size_mess));
     tmp +=  sizeof(n_size_mess);
     BCOPY(message,tmp, h_size_mess);


     if ((i = write_size(socket,buf,total_size)) == -1) {
	  perror("send_message_string: write/send");
     }
	  
     if (i != total_size) {
	  fprintf(stderr,LG_STR("Did not write all the bytes in send_message_string.\n",
				"Did not write all the bytes in send_message_string.\n"));
     }

     free(buf);

}

void send_message_string(PString message, PString rec)
{
     send_message_string_socket(mp_socket,rec,message);
}

void broadcast_message_string_socket(int socket, PString message )
{
     int i,total_size, h_size_mess;
     u_long n_size_mess;                  /* for network */
     Message_Type mt = BROADCAST_MT;
     u_long n_mt;

     char *buf, *tmp;

     h_size_mess = strlen(message);
     n_size_mess = htonl(h_size_mess);     
     n_mt = htonl(mt);

     total_size = sizeof(n_mt) + sizeof(n_size_mess) + h_size_mess;

     tmp = buf = (char *)malloc(total_size);
     
     BCOPY((char *)&n_mt,tmp, sizeof(n_mt));
     tmp +=  sizeof(n_mt);

     BCOPY((char *)&n_size_mess,tmp, sizeof(n_size_mess));
     tmp +=  sizeof(n_size_mess);
     BCOPY(message,tmp, h_size_mess);

     if ((i = write_size(socket,buf,total_size)) == -1) {
	  perror("broadcast_message_string: write/send");
     }
     
     if (i != total_size) {
	  fprintf(stderr,LG_STR("Did not write all the bytes in broadcast_message_string.\n",
				"Did not write all the bytes in broadcast_message_string.\n"));
     }

     free(buf);

}

void broadcast_message_string(PString message)
{
     broadcast_message_string_socket(mp_socket,message);
}

void multicast_message_string_socket(int socket, int nb_recs, PString *recs, PString message )
{
     int h_size_name,i,total_size, h_size_mess;
     u_long n_size_mess, n_nb_recs;                  /* for network */
     Message_Type mt = MULTICAST_MT;
     u_long n_mt;

     char *buf, *tmp;

     if (!nb_recs) return;

     h_size_name = 0;
     for (i=nb_recs; i>0; i--) {
	  h_size_name += strlen(recs[i-1]);
     }
     h_size_mess = strlen(message);
     n_nb_recs = htonl(nb_recs);
     n_size_mess = htonl(h_size_mess);
     n_mt = htonl(mt);

     total_size = sizeof(n_mt) + sizeof(n_nb_recs) + sizeof(u_long)*nb_recs + /* The number of int */
	  h_size_name + sizeof(n_size_mess) + h_size_mess;

     tmp = buf = (char *)malloc(total_size);

     BCOPY((char *)&n_mt,tmp, sizeof(n_mt));
     tmp +=  sizeof(n_mt);

     BCOPY((char *)&n_nb_recs,tmp, sizeof(n_nb_recs));
     tmp +=  sizeof(n_nb_recs);

     BCOPY((char *)&n_size_mess,tmp, sizeof(n_size_mess));
     tmp +=  sizeof(n_size_mess);
     BCOPY(message,tmp, h_size_mess);
     tmp +=  h_size_mess;

     for (i=0; i!=nb_recs; i++) {
	  int  h_rec_name = strlen(recs[i]);
	  u_long  n_rec_name = htonl(h_rec_name);

	  BCOPY((char *)&n_rec_name,tmp, sizeof(n_rec_name));
	  tmp +=  sizeof(n_rec_name);
	  BCOPY((char *)recs[i],tmp, h_rec_name);
	  tmp +=  h_rec_name;
     }

     if ((i = write_size(socket,buf,total_size)) == -1) {
	  perror("multicast_message_string: write/send");
     }
	  
     if (i != total_size) {
	  fprintf(stderr,LG_STR("Did not write all the bytes in multicast_message_string.\n",
				"N'ai pu ecrire tous les octets dans multicast_message_string.\n"));
     }

     free(buf);
}

void multicast_message_string(PString message, unsigned int nb_recs, PString *recs)
{
     if (nb_recs)
	  multicast_message_string_socket(mp_socket, nb_recs, recs, message);
}
